@@ -8,8 +8,7 @@ use Moose;
 extends 'App::AutoCRUD::Controller';
 use SQL::Abstract::More;
 use List::MoreUtils            qw/mesh firstval/;
-use Clone                      qw/clone/;
-use JSON;
+use JSON::MaybeXS ();
 use URI;
 
 use namespace::clean -except => 'meta';
@@ -108,7 +107,7 @@ sub list {
                                    $dashed_args{-page_index},
                                    $dashed_args{-page_size});
 
-  # link to update form
+  # link to update/delete forms
   $data->{where_args} = $self->_query_string(
     map { ("where.$_" => $where_args{$_}) } keys %where_args,
    );
@@ -139,10 +138,9 @@ sub _add_links_to_other_pages {
 sub id {
   my ($self, $table) = @_;
 
-  my $data = $self->descr($table);
+  my $data     = $self->descr($table);
 
   my $pk       = $data->{primary_key};
-  my %is_pk    = map {($_ => 1)} @$pk;
   my @vals     = $self->context->extract_path_segments(scalar(@$pk));
   my %criteria = mesh @$pk, @vals;
 
@@ -155,11 +153,7 @@ sub id {
 
   # links
   my %where_pk = map { ("where_pk.$_" => $criteria{$_}) } keys %criteria;
-  $data->{delete_args} = $self->_query_string(%where_pk);
-  $data->{update_args} = $self->_query_string(%where_pk);
-  my @clone_args = map  { ($_ => $row->{$_}) } 
-                   grep {!$is_pk{$_} && defined $row->{$_}} keys %$row;
-  $data->{clone_args} = $self->_query_string(@clone_args);
+  $data->{where_pk} = $self->_query_string(%where_pk);
 
   return $data;
 }
@@ -182,68 +176,90 @@ sub search {
     my @cols = split /,/, (delete $req_data->{-columns} || "");
     $req_data->{"col.$_"} = 1 foreach @cols;
     my $data = $self->descr($table);
-    my $json_maker = JSON->new();
-    $data->{init_form} = $json_maker->encode($req_data);
+    $data->{init_form} = $self->_encode_json($req_data);
     return $data;
   }
 }
 
 
-
 sub update {
   my ($self, $table) = @_;
 
+  if ($self->context->req->method eq 'POST') {
+    $self->_do_update_data($table);
+  }
+  else {
+    $self->_display_update_form($table);
+  }
+}
+
+
+sub _do_update_data {
+  my ($self, $table) = @_;
+
   my $context    = $self->context;
   my $req_data   = $context->req_data;
   my $datasource = $context->datasource;
 
-  if ($context->req->method eq 'POST') {
-    # columns to update
-    my $to_set = $req_data->{set} || {};
-    foreach my $key (keys %$to_set) {
-      my $val = $to_set->{$key};
-      delete $to_set->{$key} if ! length $val;
-      $to_set->{$key} = undef if $val eq 'Null';
-    }
-    keys %$to_set or die "nothing to update";
+  # columns to update
+  my $to_set = $req_data->{set} || {};
+  foreach my $key (keys %$to_set) {
+    my $val = $to_set->{$key};
+    delete $to_set->{$key} if ! length $val;
+    $to_set->{$key} = undef if $val eq 'Null';
+  }
+  keys %$to_set or die "nothing to update";
 
-    # filtering criteria
-    my $where  = $req_data->{where} or die "update without any '-where' clause";
-    my $criteria = $datasource->query_parser->parse($where);
-    $criteria and keys %$criteria or die "update without any '-where' criteria";
+  # build filtering criteria
+  my $where  = $req_data->{where} or die "update without any '-where' clause";
+  my $criteria = $datasource->query_parser->parse($where);
+  $criteria and keys %$criteria or die "update without any '-where' criteria";
 
-    # perform the update
-    my $db_table  = $datasource->schema->db_table($table);
-    my $n_updates = $db_table->update(-set => $to_set, -where => $criteria);
+  # perform the update
+  my $db_table  = $datasource->schema->db_table($table);
+  my $n_updates = $db_table->update(-set => $to_set, -where => $criteria);
 
-    # redirect to a list to display the results
-    my $message = ($n_updates == 1) ? "1 record was updated"
-                                    : "$n_updates records were updated";
-    # TODO: $message could repeat the $to_set pairs
-    my $query_string = $self->_query_string(%$where, -message => $message);
-    $self->redirect("list?$query_string");
+  # redirect to a list to display the results
+  my $message = ($n_updates == 1) ? "1 record was updated"
+                                  : "$n_updates records were updated";
+  # TODO: $message could repeat the $to_set pairs
+  my $query_string = $self->_query_string(%$where, -message => $message);
+  $self->redirect("list?$query_string");
+}
+
+sub _display_update_form {
+  my ($self, $table) = @_;
+
+  my $context    = $self->context;
+  my $req_data   = $context->req_data;
+  my $datasource = $context->datasource;
+  my $data       = $self->descr($table);
+
+  if (my $where_pk  = delete $req_data->{where_pk}) {
+    # we got the primary key of one single record
+    $data->{where_pk}  = $where_pk;
+    $req_data->{where} = $where_pk;
+
+    # fetch current values so that we can display them on page
+    my $criteria = $datasource->query_parser->parse($where_pk);
+    my $db_table = $datasource->schema->db_table($table);
+    $req_data->{curr} = $db_table->select(-where     => $criteria,
+                                          -result_as => 'firstrow');
   }
   else {
-    # display the update form
-    my $data = $self->descr($table);
-    my $json_maker = JSON->new->convert_blessed;
-    if (my $where_pk  = delete $req_data->{where_pk}) {
-      $data->{where_pk} = $where_pk;
-      $req_data->{where} = $where_pk;
-
-      my $criteria = $datasource->query_parser->parse($where_pk);
-      my $db_table = $datasource->schema->db_table($table);
-      $req_data->{curr} = $db_table->select(-where     => $criteria,
-                                            -result_as => 'firstrow');
-    };
-    if (my $noupd = delete $req_data->{_noupd}) {
-      # fields that should not be updatable
-      $data->{noupd}{$_} = 1 foreach split qr[/], $noupd;
-    }
-    $data->{init_form} = $json_maker->encode($req_data);
-
-    return $data;
+    # we got criteria that may touch several records
+    $self->_mark_multicols_keys($data);
   }
+
+  # fields that should not be updatable
+  if (my $noupd = delete $req_data->{_noupd}) {
+    $data->{noupd}{$_} = 1 foreach split qr[/], $noupd;
+  }
+
+  # initial values for the form
+  $data->{init_form} = $self->_encode_json($req_data);
+
+  return $data;
 }
 
 
@@ -254,7 +270,8 @@ sub delete {
   my $req_data   = $context->req_data;
   my $datasource = $context->datasource;
 
-  if ($context->req->method eq 'POST') {
+  if ($context->req->method eq 'POST') { # POST => delete in database
+    # build filtering criteria
     my $where = $req_data->{where} or die "delete without any '-where' clause";
     my $criteria = $datasource->query_parser->parse($where);
     $criteria and keys %$criteria or die "delete without any '-where' criteria";
@@ -269,21 +286,63 @@ sub delete {
     my $query_string = $self->_query_string(%$where, -message => $message);
     $self->redirect("list?$query_string");
   }
-  else {
+  else {                                  # GET => display the delete form
     # display the delete form
     my $data = $self->descr($table);
     if (my $where_pk  = delete $req_data->{where_pk}) {
+      # we got the primary key of one single record
       $data->{where_pk}  = $where_pk;
       $req_data->{where} = $where_pk;
-    };
-    my $json_maker = JSON->new();
-    $data->{init_form} = $json_maker->encode($req_data);
+    }
+    else {
+      # we got criteria that may touch several records
+      $self->_mark_multicols_keys($data);
+    }
+
+    # initial values for the form
+    $data->{init_form} = $self->_encode_json($req_data);
 
     return $data;
   }
 }
 
 
+
+
+sub clone {
+  my ($self, $table) = @_;
+
+  my $context = $self->context;
+  $context->req->method eq 'GET'
+    or die "the /clone URL only accepts GET requests";
+
+  # get primary key
+  my $data    = $self->descr($table);
+  my $pk      = $data->{primary_key};
+  my %is_pk   = map {$_ => 1} @$pk;
+  my @vals    = $context->extract_path_segments(scalar(@$pk));
+
+  # get row from database
+  my $row = $self->datasource->schema->db_table($table)->fetch(@vals);
+
+  # populate req_data before calling insert()
+  my $req_data   = $context->req_data;
+  foreach my $col (keys %$row) {
+    my $val = $row->{$col};
+    $req_data->{$col} = $val if $val and !$is_pk{$col};
+  }
+
+  # cheat with path (simulating a call to insert())
+  my $path = $context->path;
+  $path =~ s/clone$/insert/;
+  $context->set_path($path);
+  $context->set_template('table/insert.tt');
+
+  # forward to insert()
+  $self->insert($table);
+}
+
+
 sub insert {
   my ($self, $table) = @_;
 
@@ -304,14 +363,14 @@ sub insert {
   else {
     # display the insert form
     my $data = $self->descr($table);
-    my $json_maker = JSON->new();
-    $data->{init_form} = $json_maker->encode($req_data);
+    $data->{init_form} = $self->_encode_json($req_data);
 
     return $data;
   }
 }
 
 
+
 sub count_where { # used in Ajax mode by update and delete forms
   my ($self, $table) = @_;
 
@@ -353,17 +412,38 @@ sub _query_string {
     my $val = $params{$key};
     length $val or next KEY;
 
-    # cheap URI escape
+    # cheap URI escape (for chars '=', '&', ';' and '+')
     s/=/%3D/g, s/&/%26/g, s/;/%3B/g, s/\+/%2B/g for $key, $val;
 
     push @fragments, "$key=$val";
   }
-  return join "&", @fragments;
 
-  # TODO: decide about proper way to handle accented chars in URIs.
-  # URI_escape did not work because of conflicts utf8/latin1
-  # Hints : http://www.w3.org/International/articles/idn-and-iri/
-  #         L<URI/as_iri>
+  return join "&", @fragments;
+}
+
+
+sub _encode_json {
+  my ($self, $data) = @_;
+
+  # utf8-encoding is done in the view, so here we turn it off
+  my $json_maker = JSON::MaybeXS->new(allow_blessed   => 1,
+                                      convert_blessed => 1,
+                                      utf8            => 0);
+  return $json_maker->encode($data);
+}
+
+
+sub _mark_multicols_keys {
+  my ($self, $data) = @_;
+
+  if (my $sep = $self->datasource->multicols_sep) {
+    # in case of multi-columns keys, the form needs to add special fields
+    # and to ignore regular fields for those columns
+    my $where = $self->context->req_data->{where} || {};
+    my @multi_cols_keys = grep m[$sep], keys %$where;
+    $data->{multi_cols_keys} = \@multi_cols_keys;
+    $data->{ignore_col}{$_} = 1 foreach map {split m[$sep]} @multi_cols_keys;
+  }
 }
 
 
@@ -401,7 +481,7 @@ C<primary_key> (arrayref of primary key columns).
 
 Returns a list of records from the table, corresponding to the query
 parameters specified in the URL. 
-[TODO: EXPLAIN MORE]
+[TODO: EXPLAIN MORE -- in particular the "-template" arg ]
 
 
 
