@@ -9,7 +9,7 @@ use DBI;
 use Clone           qw/clone/;
 use List::MoreUtils qw/part/;
 use Scalar::Does    qw/does/;
-use SQL::Abstract::FromQuery 0.05;
+use SQL::Abstract::FromQuery 0.08;
 
 use namespace::clean -except => 'meta';
 
@@ -27,6 +27,8 @@ has 'query_parser' => (is => 'ro', isa => 'SQL::Abstract::FromQuery',
                        builder => '_query_parser', lazy => 1);
 has 'tablegroups'  => (is => 'ro', isa => 'ArrayRef',
                        builder => '_tablegroups', lazy => 1);
+has 'multicols_sep'=> (is => 'ro', isa => 'Str', default => '/');
+
 
 # indirectly generated through the _schema builder method
 has 'generated_schema' => (is => 'ro', isa => 'Str', init_arg => undef);
@@ -97,12 +99,8 @@ sub _schema {
 
     if (! $self->app->is_class_loaded($schema_class)) {
       # build a schema generator from the DBI connection
-
       my $dbh = $self->dbh;
-
       require DBIx::DataModel::Schema::Generator;
-
-
       my $generator = DBIx::DataModel::Schema::Generator->new(
         -schema => $schema_class,
        );
@@ -126,7 +124,9 @@ sub _schema {
 sub _query_parser {
   my $self = shift;
 
-  return SQL::Abstract::FromQuery->new;
+  my $sep  = $self->multicols_sep;
+  my @args = $sep ? (-multicols_sep => $sep) : ();
+  return SQL::Abstract::FromQuery->new(@args);
 }
 
 
@@ -295,6 +295,8 @@ sub colgroups {
 }
 
 
+
+
 sub _columns_from_Oracle_synonym {
   my ($self, $db_schema, $syn_name) = @_;
 
